@@ -2,7 +2,7 @@
 #
 #   strip.sh - Strip debugging symbols from binary files
 #
-#   Copyright (c) 2007-2020 Pacman Development Team <pacman-dev@archlinux.org>
+#   Copyright (c) 2007-2021 Pacman Development Team <pacman-dev@archlinux.org>
 #
 #   This program is free software; you can redistribute it and/or modify
 #   it under the terms of the GNU General Public License as published by
@@ -36,11 +36,23 @@ build_id() {
 }
 
 source_files() {
-	LANG=C readelf "$1" --debug-dump | \
-		awk '/DW_AT_name +:/{name=$NF}/DW_AT_comp_dir +:/{{if (name == "<artificial>") next}{if (name !~ /^[<\/]/) {printf "%s/", $NF}}{print name}}'
+	# This function does two things:
+	#
+	# 1) rewrites source file locations for packages not respecting prefix-
+	#    map switches.  This ensures all source file references in debug
+	#    info point to $dbgsrcdir.
+	#
+	# 2) outputs a list of files from the package source files to stdout
+	#    while stripping the $dbgsrcdir prefix
+
+	LANG=C debugedit --no-recompute-build-id \
+		--base-dir "${srcdir}" \
+		--dest-dir "${dbgsrcdir}" \
+		--list-file /dev/stdout "$1" \
+		| sort -zu | tr '\0' '\n'
 }
 
-strip_file() {
+collect_debug_symbols() {
 	local binary=$1; shift
 
 	if check_option "debug" "y"; then
@@ -58,17 +70,22 @@ strip_file() {
 		# copy source files to debug directory
 		local file dest t
 		while IFS= read -r t; do
-			file=${t/${dbgsrcdir}/"$srcdir"}
-			dest="${dbgsrc/"$dbgsrcdir"/}$t"
-			if ! [[ -f $dest ]]; then
-				mkdir -p "${dest%/*}"
+			file="${srcdir}/${t}"
+			dest="${dbgsrc}/${t}"
+			mkdir -p "${dest%/*}"
+			if [[ -f "$file" ]]; then
 				cp -- "$file" "$dest"
 			fi
 		done < <(source_files "$binary")
 
 		# copy debug symbols to debug directory
 		mkdir -p "$dbgdir/${binary%/*}"
-		objcopy --only-keep-debug "$binary" "$dbgdir/$binary.debug"
+
+		# abandon processing files that are not a recognised format
+		if ! objcopy --only-keep-debug "$binary" "$dbgdir/$binary.debug" 2>/dev/null; then
+			return
+		fi
+
 		local tempfile=$(mktemp "$binary.XXXXXX")
 		objcopy --add-gnu-debuglink="$dbgdir/${binary#/}.debug" "$binary" "$tempfile"
 		cat "$tempfile" > "$binary"
@@ -95,7 +112,10 @@ strip_file() {
 			ln -s "$target" "$dbgdir/.build-id/${bid:0:2}/${bid:2}.debug"
 		fi
 	fi
+}
 
+strip_file(){
+	local binary=$1; shift
 	local tempfile=$(mktemp "$binary.XXXXXX")
 	if strip "$@" "$binary" -o "$tempfile"; then
 		cat "$tempfile" > "$binary"
@@ -103,6 +123,16 @@ strip_file() {
 	rm -f "$tempfile"
 }
 
+strip_lto() {
+	local binary=$1;
+
+	local tempfile=$(mktemp "$binary.XXXXXX")
+	if strip -R .gnu.lto_* -R .gnu.debuglto_* -N __gnu_lto_v1 "$binary" -o "$tempfile"; then
+		cat "$tempfile" > "$binary"
+	fi
+	rm -f "$tempfile"
+}
+
 
 tidy_strip() {
 	if check_option "strip" "y"; then
@@ -114,33 +144,37 @@ tidy_strip() {
 		if check_option "debug" "y"; then
 
 			dbgdir="$pkgdirbase/$pkgbase-debug/usr/lib/debug"
-			dbgsrcdir="${DBGSRCDIR:-/usr/src/debug}"
+			dbgsrcdir="${DBGSRCDIR:-/usr/src/debug}/${pkgbase}"
 			dbgsrc="$pkgdirbase/$pkgbase-debug$dbgsrcdir"
 			mkdir -p "$dbgdir" "$dbgsrc"
 		fi
 
 		local binary strip_flags
-		find . -type f -perm -u+w -print0 2>/dev/null | while IFS= read -rd '' binary ; do
-			case "$(file -S -bi "$binary")" in
-				*application/x-sharedlib*)  # Libraries (.so)
+		find . -type f -perm -u+w -print0 2>/dev/null | LC_ALL=C sort -z | while IFS= read -rd '' binary ; do
+			local STATICLIB=0
+			case "$(LC_ALL=C readelf -h "$binary" 2>/dev/null)" in
+				*Type:*'DYN (Shared object file)'*) # Libraries (.so) or Relocatable binaries
 					strip_flags="$STRIP_SHARED";;
-				*application/x-archive*)    # Libraries (.a)
-					strip_flags="$STRIP_STATIC";;
-				*application/x-object*)
-					case "$binary" in
-						*.ko)                   # Kernel module
-							strip_flags="$STRIP_SHARED";;
-						*)
-							continue;;
-					esac;;
-				*application/x-executable*) # Binaries
+				*Type:*'DYN (Position-Independent Executable file)'*) # Relocatable binaries
+					strip_flags="$STRIP_SHARED";;
+				*Type:*'EXEC (Executable file)'*) # Binaries
 					strip_flags="$STRIP_BINARIES";;
-				*application/x-pie-executable*)  # Relocatable binaries
-					strip_flags="$STRIP_SHARED";;
+				*Type:*'REL (Relocatable file)'*) # Libraries (.a) or objects
+					if ar t "$binary" &>/dev/null; then # Libraries (.a)
+						strip_flags="$STRIP_STATIC"
+						STATICLIB=1
+					elif [[ $binary = *'.ko' || $binary = *'.o' ]]; then # Kernel module or object file
+						strip_flags="$STRIP_SHARED"
+					else
+						continue
+					fi
+					;;
 				*)
 					continue ;;
 			esac
+			(( ! STATICLIB )) && collect_debug_symbols "$binary"
 			strip_file "$binary" ${strip_flags}
+			(( STATICLIB )) && strip_lto "$binary"
 		done
 	fi
 }
